import type { Message, ServerOptions } from '../interfaces/protocol';
import type { SelfDebug } from './self-debug';
export declare type ConnectionEventMap<TExtensionServerOptions, TSlowUpdateExtensionData> = {
    open: (e: Event) => void;
    message: (data: Message<TExtensionServerOptions, TSlowUpdateExtensionData>, e: MessageEvent) => void;
    error: (e: ErrorEvent) => void;
    close: (e: CloseEvent) => void;
};
export declare class Connection<TExtensionServerOptions, TSlowUpdateExtensionData> {
    #private;
    constructor(url: string, selfDebug: SelfDebug | null);
    on<TKey extends keyof ConnectionEventMap<TExtensionServerOptions, TSlowUpdateExtensionData>>(key: TKey, handler: ConnectionEventMap<TExtensionServerOptions, TSlowUpdateExtensionData>[TKey]): void;
    off<TKey extends keyof ConnectionEventMap<TExtensionServerOptions, TSlowUpdateExtensionData>>(key: TKey, handler: ConnectionEventMap<TExtensionServerOptions, TSlowUpdateExtensionData>[TKey]): void;
    sendReplaceText(start: number, length: number, newText: string, cursorIndexAfter: number, reason?: string | null): Promise<void>;
    sendMoveCursor(cursorIndex: number): Promise<void>;
    sendTypeChar(char: string): Promise<void>;
    sendCompletionState(indexOrCommand: 'info' | 'force' | number | 'cancel', indexIfInfo?: number): Promise<void>;
    sendSignatureHelpState(command: 'force' | 'cancel'): Promise<void>;
    sendRequestInfoTip(cursorIndex: number): Promise<void>;
    sendSlowUpdate(): Promise<void>;
    sendApplyDiagnosticAction(actionId: number): Promise<void>;
    sendSetOptions(options: ServerOptions | Partial<ServerOptions & TExtensionServerOptions>): Promise<void>;
    sendRequestSelfDebugData(): Promise<void>;
    close(): void;
}
